#!/usr/bin/env python
# coding: utf-8

import requests
import logging

import trivoreid.utils.service_utils as su
from trivoreid.utils.criteria import Filter
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class SMSService(object):
    '''
    Class to wrap /sms Trivore ID API.
    '''

    _SEND = 'api/rest/v1/sms/send'
    _REGIONS = 'api/rest/v1/sms/regions'
    _SEND_TO_USER = 'api/rest/v1/user/{}/sms/send'

    def __init__(self, credentials,
                 group_service=None,
                 user_service=None):
        self._group_service = group_service
        self._user_service = user_service
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._auth_header = credentials.access_token

    def send(self, message):
        '''Send SMS message.
        Args:
            params (SMSMessage)    : SMS message parameters
        Returns:
            Response body
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the Email is wrong
        '''
        if 'SMSMessage' not in str(type(message)):
            raise TrivoreIDSDKException('SMSMessage type is wrong!')

        response = self._session.post(su.uri(self._server, self._SEND),
                                      json=message.serialize(),
                                headers=self._auth_header,
                                auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully sent SMS message to a mobile number {}.'
                                .format(message.to))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def send_to_user(self, userId, message):
        '''Send SMS message.
        Args:
            userId (str)           : user unique identifier
            message (SMSMessage)   : SMS message parameters
        Returns:
            Response body
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the Email is wrong
        '''
        if 'SMSMessage' not in str(type(message)):
            raise TrivoreIDSDKException('SMSMessage type is wrong!')

        response = self._session.post(
                    su.uri(self._server, self._SEND_TO_USER).format(userId),
                    json=message.serialize(),
                    headers=self._auth_header,
                    auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully sent SMS message.')
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_supported_regions(self):
        '''Get list of all supported regions.
        Returns:
            Regions object
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        response = self._session.get(su.uri(self._server, self._REGIONS),
                                     headers=self._auth_header,
                                     auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully got the list of regions')
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def send_to_group_members(self,
                              nsCode,
                              message,
                              *groupIdOrName):
        '''
        Send SMS message to all users belonging to the groups.
        Args:
            nsCode (str)         : namespace code
            message (SMSMessage) : SMS message to send
            groupIdOrName        : group names or ids. Max. 20 groups.
        Returns:
            Rest with responses
        '''
        if len(groupIdOrName) > 20:
            raise TrivoreIDSDKException('Limit of 20 groups was exceeded')

        filt = Filter(Filter.EQUAL, 'name' , groupIdOrName[0])
        for i in range(1, len(groupIdOrName)):
            filt = Filter().or_filters(filt,
                                Filter(Filter.EQUAL, 'name' , groupIdOrName[i]))
            filt = Filter().or_filters(filt,
                                Filter(Filter.EQUAL, 'id' , groupIdOrName[i]))

        groups = self._group_service.get_all(filt)
        groupIDs = []
        for g in groups:
            groupIDs.append(g.id)

        filt2 = Filter(Filter.EQUAL, 'memberOf' , groupIDs[0])

        for i in range(1, len(groupIDs)):
            filt2 = Filter().or_filters(filt2,
                                Filter(Filter.EQUAL, 'memberOf' , groupIDs[i]))

        users = self._user_service.get_all(filt2)

        mobiles_list = []
        for user in users:
            if len(user.mobiles) > 0:
                mobiles_list.append(user.mobiles[0].number)

        responses = []
        for mobile in mobiles_list:
            message.to = mobile
            responses.append(self.send(message))
        return responses



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.user import Consents, Address

class OIDCUser(object):
    '''
    OIDC User wrapper.
    '''

    _NAMESPACE_CLAIM = 'https://oneportal.trivore.com/claims/namespace'
    _GROUPS_CLAIM = 'https://oneportal.trivore.com/claims/groups'
    _CONCENTS_CLAIM = 'https://oneportal.trivore.com/claims/consents'
    _MINOR_CLAIM = 'https://oneportal.trivore.com/claims/minor'

    def __init__(self, data):
        '''
        Args:
            data (dict): user fields
        '''

        self.id = data.pop('sub', None)
        self.email = data.pop('email', None)
        self.email_verified = data.pop('email_verified', None)
        self.phone = data.pop('phone_number', None)
        self.phone_number_verified = data.pop('phone_number_verified', None)
        self.preferred_username = data.pop('preferred_username', None)
        self.groups = data.pop(self._GROUPS_CLAIM, None)
        self.nsCode = data.pop(self._NAMESPACE_CLAIM, None)
        self.minor = data.pop(self._MINOR_CLAIM, None)
        self.consents = Consents(data.pop(self._CONCENTS_CLAIM, {}))
        self.locale = data.pop('locale', None)
        self.nickname = data.pop('nickname', None)
        self.zoneinfo = data.pop('zoneinfo', None)

        address = data.pop('address', {})
        self.address = Address()
        self.address.country = address.pop('country', None)
        self.address.locality = address.pop('locality', None)
        self.address.region = address.pop('region', None)
        self.address.postalCode = address.pop('postal_code', None)
        self.address.streetAddress = address.pop('street_address', None)

    def serialize(self):
        return {
            'id' : self.id,
            'email' : self.email,
            'email_verified' : self.email_verified,
            'phone' : self.phone,
            'phone_number_verified' : self.phone_number_verified,
            'address' : self.address.serialize(),
            'preferred_username' : self.preferred_username,
            'groups' : self.groups,
            'nsCode' : self.nsCode,
            'minor' : self.minor,
            'consents' : self.consents.serialize(),
            'locale' : self.locale,
            'nickname' : self.nickname,
            'zoneinfo' : self.zoneinfo
        }



#!/usr/bin/env python
# coding: utf-8

from trivoreid.exceptions import TrivoreIDSDKException

class Filter:
    '''
    Object to generate the query filter.
    '''

    EQUAL = ' eq '
    NOT_EQUAL = ' ne '
    CONTAINS = ' co '
    STARTS_WITH = ' sw '
    ENDS_WITH = ' ew '
    PRESENT = ' pr '
    GREATER_THAN = ' gt '
    GREATER_OR_EQUAL_THAN = ' ge '
    LESS_THAN = ' lt '
    LESS_OR_EQUAL_THAN = ' le '
    _AND = ' and '
    _OR = ' or '

    def __init__(self,
                 filter_type = None,
                 attribute = None,
                 value = None):
        '''
        Args:
            filter_type (Filter) : enum from the Filter class
            attribute (str)      : the name of the attribute
            value                : value of the attribute
        '''
        self.filter_type = filter_type
        self.attribute = attribute
        self.value = value

    def and_filters(self, filter1, filter2):
        '''
        Combine two filters.
        Args:
            filter1 (Filter)
            filter2 (Filter)
        '''
        self._check_filter_type(filter1)
        self._check_filter_type(filter2)
        return Filter(Filter._AND, None, [filter1, filter2])

    def or_filters(self, filter1, filter2):
        '''
        Combine two filters.
        Args:
            filter1 (Filter)
            filter2 (Filter)
        '''
        self._check_filter_type(filter1)
        self._check_filter_type(filter2)
        return Filter(Filter._OR, None, [filter1, filter2])

    def generate(self):
        '''
        Generate filter parameter.
        Returns:
            Filter string.
        '''
        if type(self.value) == list:
            if len(self.value) != 2:
                raise TrivoreIDSDKException('Filter contains {} values'
                                            .format(len(self.value)))
            return "({}) {} ({})".format(self.value[0].generate(),
                                         self.filter_type,
                                         self.value[1].generate())
        elif 'Filter' in str(type(self.value)):
            return self.value.generate()
        else:
            if self.filter_type == self.PRESENT:
                return (self.attribute + self.filter_type)
            elif type(self.value) is int:
                return (self.attribute + self.filter_type + str(self.value))
            elif type(self.value) is bool:
                if self.value:
                    return (self.attribute + self.filter_type + 'true')
                else:
                    return (self.attribute + self.filter_type + 'false')
            else:
                return (self.attribute + self.filter_type
                                                + '"' + str(self.value) + '"')

    def _check_filter_type(self, f):
        if 'Filter' not in str(type(f)):
            raise TrivoreIDSDKException('Filter type is wrong!')



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.misc import Meta

class Group(object):
    '''
    Class that represents a Group object.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict)         : group fields
        Dictionary keys:
            'id' (str) - group ID
            'name' (str) - group name
            'description' (str) - group description
            'nsCode' (str) - namespace code. Use only when creating group, must
                             be among allowed Namespaces.
            'disallowUsers' (bool) disallow user accounts
            'disallowContacts' (bool) disallow contacts
            'disallowGroups' (bool) disallow groups
            'memberOf' (list) set of Group IDs this group is member of
            'meta' (dict) meta data
        '''
        self.id = data.pop('id', None)
        self.name = data.pop('name', None)
        self.description = data.pop('description', None)
        self.nsCode = data.pop('nsCode', None)
        self.disallowUsers = data.pop('disallowUsers', None)
        self.disallowContacts = data.pop('disallowContacts', None)
        self.disallowGroups = data.pop('disallowGroups', None)
        self.memberOf = data.pop('memberOf', [])
        self.meta = Meta(data.pop('meta', {}))

    def serialize(self):
        return {
            'id'                : self.id,
            'name'              : self.name,
            'description'       : self.description,
            'nsCode'            : self.nsCode,
            'disallowUsers'     : self.disallowUsers,
            'disallowContacts'  : self.disallowContacts,
            'disallowGroups'    : self.disallowGroups,
            'memberOf'          : self.memberOf,
            'meta'              : self.meta.serialize()
        }



#!/usr/bin/env python
# coding: utf-8

import os
import requests

import trivoreid.utils.service_utils as su

from trivoreid.oidc.oidc_user import OIDCUser

from trivoreid.services.user_service import UserService
from trivoreid.services.namespace_service import NamespaceService
from trivoreid.services.group_service import GroupService
from trivoreid.services.datastorage_service import DataStorageService
from trivoreid.services.mydata_service import MyDataService
from trivoreid.services.profile_service import ProfileService
from trivoreid.services.email_service import EmailService
from trivoreid.services.email_templates_service import EmailTemplatesService
from trivoreid.services.sms_service import SMSService

class TrivoreID:
    '''
    Main class to perform REST API requests to TrivoreID.
    '''

    def __init__(self,
                 path='/etc/trivoreid/client_sdk.properties',
                 oauth=None,
                 access_token=None,
                 server=None,
                 client_id=None,
                 client_secret=None):
        '''
        Args:
            path (str)               : path to the properties file with the
                                       Management API credentials if OAuth was
                                       not used. Default:
                                       /etc/trivoreid/client_sdk.properties
            oauth (OAuth2Session)    : if None, then ManagementAPI credentials
                                       or access token will be used,
            access_token (str)       : when the password grant authorisation
                                       was used.
            server (str)             : server URL. If None, then value from
                                       properties file will be used.
            client_id (str)          : Management API client id. If None, then
                                       value from properties file will be used.
            client_secret (str)      : Management API client secret. If None,
                                       then value from properties file will be
                                       used.

        Example of the properties file:
            service.address=<placeholder>
            mgmtapi.id=<placeholder>
            mgmtapi.secret=<placeholder>
        '''

        USER_INFO = 'openid/userinfo'

        if os.path.exists(path):
            properties = su.get_properties(path)
        else:
            properties = {}

        if server is None:
            server = properties.pop('service.address', None)

        if server[-1] != '/':
            server += '/'

        self.oidc_user = None

        if access_token is not None:
            access_token = {'Authorization': 'Bearer ' + access_token}
            response = requests.get(server + USER_INFO, headers=access_token)
            if response.status_code is 200:
                self.oidc_user = OIDCUser(response.json())

        if oauth is None and access_token is None:
            if client_id is None:
                client_id = properties.pop('mgmtapi.id', None)
            if client_secret is None:
                client_secret = properties.pop('mgmtapi.secret', None)
        elif oauth is not None:
            response = oauth.get(server + USER_INFO)
            if response.status_code == 200:
                self.oidc_user = OIDCUser(response.json())

        if client_id is None and client_secret is None:
            auth = None
        else:
            auth = (client_id, client_secret)

        creds = self._Credentials(auth, server, oauth, access_token)

        self.group_service = GroupService(creds)
        self.namespace_service = NamespaceService(creds, self.group_service)
        self.user_service = UserService(creds, self.group_service)
        self.datastorage_service = DataStorageService(creds)
        self.mydata_service = MyDataService(creds)
        self.profile_service = ProfileService(creds, self.oidc_user)
        self.email_service = EmailService(creds, self.group_service,
                                                        self.user_service)
        self.email_templates_service = EmailTemplatesService(creds)
        self.sms_service = SMSService(creds, self.group_service,
                                                        self.user_service)

    class _Credentials(object):

        def __init__(self, auth, server, oidc_client, access_token):

            self.auth = auth
            self.server = server
            self.oidc_client = oidc_client
            self.access_token = access_token
